// Portions of this work are Copyright 2018 The Time Machine Authors. All rights reserved.
// Portions of this work are Copyright 2018 The Noda Time Authors. All rights reserved.
// Use of this source code is governed by the Apache License 2.0, as found in the LICENSE.txt file.

import 'dart:async';

import 'package:time_machine/src/time_machine_internal.dart';
import 'package:test/test.dart';

import 'time_machine_testing.dart';

Future main() async {
  await runTests();
}

@Test()
void Addition_WithPeriod()
{
  LocalDate start = new LocalDate(2010, 6, 19);
  Period period = new Period.fromMonths(3) + new Period.fromDays(10);
  LocalDate expected = new LocalDate(2010, 9, 29);
  expect(expected, start + period);
}

@Test()
void Addition_TruncatesOnShortMonth()
{
  LocalDate start = new LocalDate(2010, 1, 30);
  Period period = new Period.fromMonths(1);
  LocalDate expected = new LocalDate(2010, 2, 28);
  expect(expected, start + period);
}

@Test()
void Addition_WithNullPeriod_ThrowsArgumentNullException()
{
  LocalDate date = new LocalDate(2010, 1, 1);
  // Call to ToString just to make it a valid statement
  // Assert.Throws<ArgumentNullException>
  Period p = null;
  expect(() => (date + p).toString(), throwsArgumentError);
}

@Test()
void Subtraction_WithPeriod()
{
  LocalDate start = new LocalDate(2010, 9, 29);
  Period period = new Period.fromMonths(3) + new Period.fromDays(10);
  LocalDate expected = new LocalDate(2010, 6, 19);
  expect(expected, start - period);
}

@Test()
void Subtraction_TruncatesOnShortMonth()
{
  LocalDate start = new LocalDate(2010, 3, 30);
  Period period = new Period.fromMonths(1);
  LocalDate expected = new LocalDate(2010, 2, 28);
  expect(expected, start - period);
}

@Test()
void Subtraction_WithNullPeriod_ThrowsArgumentNullException()
{
  LocalDate date = new LocalDate(2010, 1, 1);
  // Call to ToString just to make it a valid statement
  // Assert.Throws<ArgumentNullException>
  Period p = null;
  expect(() => (date - p).toString(), willThrow<TypeError>());
}

@Test()
void Addition_PeriodWithTime()
{
  LocalDate date = new LocalDate(2010, 1, 1);
  Period period = new Period.fromHours(1);
  // Use method not operator here to form a valid statement
  // Assert.Throws<ArgumentException>
  expect(() => LocalDate.add(date, period), throwsArgumentError);
}

@Test()
void Subtraction_PeriodWithTime()
{
  LocalDate date = new LocalDate(2010, 1, 1);
  Period period = new Period.fromHours(1);
  // Use method not operator here to form a valid statement
  // Assert.Throws<ArgumentException>
  expect(() => LocalDate.subtract(date, period), throwsArgumentError);
}

@Test()
void PeriodAddition_MethodEquivalents()
{
  LocalDate start = new LocalDate(2010, 6, 19);
  Period period = new Period.fromMonths(3) + new Period.fromDays(10);
  expect(start + period, LocalDate.add(start, period));
  expect(start + period, start.plus(period));
}

@Test()
void PeriodSubtraction_MethodEquivalents()
{
  LocalDate start = new LocalDate(2010, 6, 19);
  Period period = new Period.fromMonths(3) + new Period.fromDays(10);
  LocalDate end = start + period;
  expect(start - period, LocalDate.subtract(start, period));
  expect(start - period, start.minusPeriod(period));
  expect(period, end - start);
  expect(period, LocalDate.between(end, start)); // LocalDate.Minus(end, start)
  expect(period, end.minusDate(start));
}
